from ftw.builder import Builder
from ftw.builder import create
from ftw.events.restapi.content import SerializeEventListingBlockToJson
from ftw.events.testing import FUNCTIONAL_TESTING
from ftw.events.tests import FunctionalTestCase
from ftw.testbrowser import browsing


class TestRestApiIntegration(FunctionalTestCase):

    layer = FUNCTIONAL_TESTING

    def setUp(self):
        super(TestRestApiIntegration, self).setUp()
        self.grant('Manager', 'Site Administrator')

        self.layer['portal'].manage_permission(
            'plone.restapi: Use REST API',
            roles=['Anonymous']
        )

        self.eventfolder = create(Builder('event folder'))
        self.eventlistingblock = self.eventfolder.objectValues()[0]
        self.event = create(Builder('event page')
                            .within(self.eventfolder)
                            .titled(u'Some Event'))

    def test_query_is_from_event_listing_block(self):
        self.assertDictEqual(
            self.eventlistingblock.restrictedTraverse('@@events').get_query(),
            SerializeEventListingBlockToJson(self.eventlistingblock, self.eventlistingblock.REQUEST).get_query()
        )

    @browsing
    def test_do_not_serialize_events_by_default_on_block(self, browser):
        browser.open(self.eventlistingblock.absolute_url(), method='GET',
                     headers={'Accept': 'application/json'})
        self.assertNotIn('items', browser.json)

    @browsing
    def test_eventlistingblock_has_block_configuration(self, browser):
        browser.open(self.eventlistingblock.absolute_url(), method='GET',
                     headers={'Accept': 'application/json'})
        self.assertIn('block-configuration', browser.json)

    @browsing
    def test_include_events(self, browser):
        browser.open(self.eventlistingblock.absolute_url() + '?include_items=true', method='GET',
                     headers={'Accept': 'application/json'})
        self.assertIn('items', browser.json)

        event = browser.json['items'][0]
        self.assertEquals(self.event.Title().decode('utf-8'), event['title'])

    @browsing
    def test_pagination(self, browser):

        for number in range(0, 5):
            create(Builder('event page')
                   .within(self.eventfolder)
                   .titled(u'Event Nr. {}'.format(number)))
        browser.open(self.eventlistingblock.absolute_url() + '?include_items=true&b_size=2', method='GET',
                     headers={'Accept': 'application/json'})

        self.assertEquals(6, browser.json['items_total'])
        self.assertIn('batching', browser.json)

    @browsing
    def test_serialize_event_by_default_on_eventfolder(self, browser):
        browser.open(self.eventfolder.absolute_url(), method='GET',
                     headers={'Accept': 'application/json'})
        self.assertIn('items', browser.json)



from ftw.events.interfaces import IEventListingBlock
from ftw.simplelayout.interfaces import IBlockConfiguration
from ftw.simplelayout.restapi.content import PersistenceDecoder
from plone import api
from plone.restapi.batching import HypermediaBatch
from plone.restapi.deserializer import boolean_value
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.dxcontent import SerializeToJson
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import Interface
from zope.interface import implementer
import json


@implementer(ISerializeToJson)
@adapter(IEventListingBlock, Interface)
class SerializeEventListingBlockToJson(SerializeToJson):
    def __call__(self, version=None, include_items=False):
        result = super(SerializeEventListingBlockToJson, self).__call__(version=version)

        include_items = self.request.form.get("include_items", include_items)
        include_items = boolean_value(include_items)
        if include_items:
            catalog = api.portal.get_tool('portal_catalog')

            brains = catalog.searchResults(**self.get_query())

            original = self.request.get('ACTUAL_URL')
            self.request['ACTUAL_URL'] = self.context.absolute_url()
            batch = HypermediaBatch(self.request, brains)

            if not self.request.form.get("fullobjects"):
                result["@id"] = batch.canonical_url
            result["items_total"] = batch.items_total
            if batch.links:
                result["batching"] = batch.links

            if "fullobjects" in list(self.request.form):
                result["items"] = [
                    getMultiAdapter(
                        (brain.getObject(), self.request), ISerializeToJson
                    )()
                    for brain in batch
                ]
            else:
                result["items"] = [
                    getMultiAdapter((brain, self.request), ISerializeToJsonSummary)()
                    for brain in batch
                ]

            self.request['ACTUAL_URL'] = original

        result['block-configuration'] = json.loads(json.dumps(
            IBlockConfiguration(self.context).load(),
            cls=PersistenceDecoder)
        )

        return result

    def get_query(self):
        return self.context.restrictedTraverse('@@events').get_query()
